@@ -23,12 +23,17 @@ namespace GuestProxyAgentTest
             var testConfigFilePath = args[0];
             var testResultFolder = args[1];
             var guestProxyAgentZipFilePath = args[2];
+            var test_arm64 = false;
+            if (args.Length > 3 && args[3].Equals("arm64", StringComparison.InvariantCultureIgnoreCase))
+            {
+                test_arm64 = true;
+            }
 
             TestCommonUtilities.TestSetup(guestProxyAgentZipFilePath, testConfigFilePath, testResultFolder);
 
             VMHelper.Instance.CleanupOldTestResourcesAndForget();
 
-            await new GuestProxyAgentScenarioTests().StartAsync(TestMapReader.ReadFlattenTestScenarioSettingFromTestMap());
+            await new GuestProxyAgentScenarioTests().StartAsync(TestMapReader.ReadFlattenTestScenarioSettingFromTestMap(test_arm64));
             Console.WriteLine("E2E Test run completed.");
         }
     }
