@@ -79,24 +79,22 @@
 //! assert_eq!(0, provision_state.1.len());
 //! ```
 
-use crate::common::{
-    config, constants, error::Error, helpers, hyper_client, logger, result::Result,
-};
+use crate::common::{config, helpers, logger};
+use crate::key_keeper::{DISABLE_STATE, UNKNOWN_STATE};
 use crate::proxy_agent_status;
 use crate::shared_state::agent_status_wrapper::{AgentStatusModule, AgentStatusSharedState};
 use crate::shared_state::key_keeper_wrapper::KeyKeeperSharedState;
 use crate::shared_state::provision_wrapper::ProvisionSharedState;
 use crate::shared_state::telemetry_wrapper::TelemetrySharedState;
 use crate::telemetry::event_reader::EventReader;
-use proxy_agent_shared::misc_helpers;
+use proxy_agent_shared::logger::LoggerLevel;
 use proxy_agent_shared::telemetry::event_logger;
-use serde_derive::{Deserialize, Serialize};
-use std::collections::HashMap;
-use std::net::Ipv4Addr;
+use proxy_agent_shared::{misc_helpers, proxy_agent_aggregate_status};
 use std::path::PathBuf;
 use std::time::Duration;
 use tokio_util::sync::CancellationToken;
 
+const PROVISION_TAG_FILE_NAME: &str = "provisioned.tag";
 const STATUS_TAG_TMP_FILE_NAME: &str = "status.tag.tmp";
 const STATUS_TAG_FILE_NAME: &str = "status.tag";
 
@@ -134,29 +132,25 @@ bitflags::bitflags! {
     }
 }
 
-/// Provision status
-/// finished - provision finished or timedout
-///            true means provision finished or timedout, false means provision still in progress
-/// errorMessage - provision error message
-#[derive(Serialize, Deserialize)]
-#[allow(non_snake_case)]
-pub struct ProvisionState {
-    pub finished: bool,
-    pub errorMessage: String,
+/// Provision internal state
+/// It is used to represent the provision state within GPA service
+/// finished_time_tick - provision finished or timedout time tick, 0 means provision still in progress
+/// error_message - provision error message
+/// key_keeper_secure_channel_state - key keeper secure_channel state
+#[derive(Clone, Debug)]
+pub struct ProvisionStateInternal {
+    pub finished_time_tick: i128,
+    pub error_message: String,
+    pub key_keeper_secure_channel_state: String,
 }
 
-impl ProvisionState {
-    pub fn new(finished: bool, error_message: String) -> Self {
-        ProvisionState {
-            finished,
-            errorMessage: error_message,
-        }
+impl ProvisionStateInternal {
+    pub fn is_secure_channel_latched(&self) -> bool {
+        self.key_keeper_secure_channel_state != DISABLE_STATE
+            && self.key_keeper_secure_channel_state != UNKNOWN_STATE
     }
 }
 
-/// Provision URL path, it is used to query the provision status from GPA service http listener
-pub const PROVISION_URL_PATH: &str = "/provision";
-
 /// Update provision state when redirector provision finished
 /// It could  be called by redirector module
 pub async fn redirector_ready(
@@ -345,7 +339,6 @@ pub async fn start_event_threads(
                 config::get_events_dir(),
                 Duration::default(),
                 config::get_max_event_file_count(),
-                logger::AGENT_LOGGER_KEY,
                 move |status: String| {
                     let cloned_agent_status_shared_state = cloned_agent_status_shared_state.clone();
                     async move {
@@ -385,7 +378,7 @@ pub async fn start_event_threads(
     tokio::spawn({
         let agent_status_task = proxy_agent_status::ProxyAgentStatusTask::new(
             Duration::from_secs(60),
-            config::get_logs_dir(),
+            proxy_agent_aggregate_status::get_proxy_agent_aggregate_status_folder(),
             cancellation_token.clone(),
             key_keeper_shared_state.clone(),
             agent_status_shared_state.clone(),
@@ -409,7 +402,7 @@ async fn write_provision_state(
     agent_status_shared_state: AgentStatusSharedState,
 ) {
     let provision_dir = provision_dir.unwrap_or_else(config::get_keys_dir);
-    let provisioned_file: PathBuf = provision_dir.join("provisioned.tag");
+    let provisioned_file: PathBuf = provision_dir.join(PROVISION_TAG_FILE_NAME);
     if let Err(e) = misc_helpers::try_create_folder(&provision_dir) {
         logger::write_error(format!("Failed to create provision folder with error: {e}"));
         return;
@@ -422,7 +415,7 @@ async fn write_provision_state(
         logger::write_error(format!("Failed to write provisioned file with error: {e}"));
     }
 
-    let failed_state_message =
+    let mut failed_state_message =
         get_provision_failed_state_message(provision_shared_state, agent_status_shared_state).await;
 
     #[cfg(not(windows))]
@@ -434,6 +427,20 @@ async fn write_provision_state(
         }
     }
 
+    if !failed_state_message.is_empty() {
+        // escape xml characters to allow the message to able be composed into xml payload
+        failed_state_message = helpers::xml_escape(failed_state_message);
+
+        // write provision failed error message to event
+        event_logger::write_event(
+            LoggerLevel::Error,
+            failed_state_message.to_string(),
+            "write_provision_state",
+            "provision",
+            logger::AGENT_LOGGER_KEY,
+        );
+    }
+
     let status_file: PathBuf = provision_dir.join(STATUS_TAG_TMP_FILE_NAME);
     match std::fs::write(status_file, failed_state_message.as_bytes()) {
         Ok(_) => {
@@ -503,99 +510,152 @@ async fn get_provision_failed_state_message(
 /// Get provision state
 /// It returns the current GPA serice provision state (from shared_state) for GPA service
 /// This function is designed and invoked in GPA service
-pub async fn get_provision_state(
+pub async fn get_provision_state_internal(
     provision_shared_state: ProvisionSharedState,
     agent_status_shared_state: AgentStatusSharedState,
-) -> ProvisionState {
-    ProvisionState {
-        finished: provision_shared_state
+    key_keeper_shared_state: KeyKeeperSharedState,
+) -> ProvisionStateInternal {
+    ProvisionStateInternal {
+        finished_time_tick: provision_shared_state
             .get_provision_finished()
             .await
-            .unwrap_or(false),
-        errorMessage: get_provision_failed_state_message(
+            .unwrap_or(0),
+        error_message: get_provision_failed_state_message(
             provision_shared_state,
             agent_status_shared_state,
         )
         .await,
+        key_keeper_secure_channel_state: key_keeper_shared_state
+            .get_current_secure_channel_state()
+            .await
+            .unwrap_or(UNKNOWN_STATE.to_string()),
     }
 }
 
-/// Provision query
+/// provision query module designed for GPA command line, serves for --status [--wait seconds] option
 /// It is used to query the provision status from GPA service via http request
-/// This struct is designed for GPA command line, serves for --status [--wait seconds] option
-pub struct ProvisionQuery {
-    port: u16,
-    wait_duration: Option<Duration>,
-}
+pub mod provision_query {
+    use crate::common::{constants, error::Error, helpers, hyper_client, logger, result::Result};
+    use proxy_agent_shared::misc_helpers;
+    use serde_derive::{Deserialize, Serialize};
+    use std::{collections::HashMap, net::Ipv4Addr, time::Duration};
 
-impl ProvisionQuery {
-    pub fn new(port: u16, wait_duration: Option<Duration>) -> ProvisionQuery {
-        ProvisionQuery {
-            port,
-            wait_duration,
+    /// Provision URL path, it is used to query the provision status from GPA service http listener
+    pub const PROVISION_URL_PATH: &str = "/provision";
+
+    /// Provision status
+    /// finished - provision finished or timedout
+    ///            true means provision finished or timedout, false means provision still in progress
+    /// errorMessage - provision error message
+    #[derive(Serialize, Deserialize)]
+    #[allow(non_snake_case)]
+    pub struct ProvisionState {
+        pub finished: bool,
+        pub errorMessage: String,
+    }
+
+    impl ProvisionState {
+        pub fn new(finished: bool, error_message: String) -> ProvisionState {
+            ProvisionState {
+                finished,
+                errorMessage: error_message,
+            }
         }
     }
 
-    /// Get current GPA service provision status and wait until the GPA service provision finished or timeout
-    /// This function is designed for GPA command line, serves for --status [--wait seconds] option
-    pub async fn get_provision_status_wait(&self) -> ProvisionState {
-        loop {
-            let state = match self.get_current_provision_status().await {
-                Ok(state) => state,
-                Err(e) => {
-                    println!(
-                        "Failed to query the current provision state with error: {}.",
-                        e
-                    );
-                    ProvisionState::new(false, String::new())
-                }
-            };
-            if state.finished {
-                return state;
-            }
-
-            if let Some(d) = self.wait_duration {
-                if d.as_millis() >= helpers::get_elapsed_time_in_millisec() {
-                    tokio::time::sleep(Duration::from_millis(100)).await;
-                    continue;
-                }
-            }
-
-            // wait timedout return as 'not finished' with empty message
-            return ProvisionState::new(false, String::new());
-        }
+    /// Provision query
+    /// It is used to query the provision status from GPA service via http request
+    /// This struct is designed for GPA command line, serves for --status [--wait seconds] option
+    pub struct ProvisionQuery {
+        port: u16,
+        wait_duration: Option<Duration>,
+        query_time_tick: i128,
     }
 
-    // Get current provision status from GPA service via http request
-    // return value
-    //  bool - true provision finished; false provision not finished
-    //  String - provision error message, empty means provision success or provision failed.
-    async fn get_current_provision_status(&self) -> Result<ProvisionState> {
-        let provision_url: String = format!(
-            "http://{}:{}{}",
-            Ipv4Addr::LOCALHOST,
-            self.port,
-            PROVISION_URL_PATH
-        );
+    impl ProvisionQuery {
+        pub fn new(port: u16, wait_duration: Option<Duration>) -> ProvisionQuery {
+            ProvisionQuery {
+                port,
+                wait_duration,
+                query_time_tick: misc_helpers::get_date_time_unix_nano(),
+            }
+        }
 
-        let provision_url: hyper::Uri = provision_url
-            .parse::<hyper::Uri>()
-            .map_err(|e| Error::ParseUrl(provision_url, e.to_string()))?;
+        #[cfg(test)]
+        pub fn get_query_time_tick(&self) -> i128 {
+            self.query_time_tick
+        }
 
-        let mut headers = HashMap::new();
-        headers.insert(constants::METADATA_HEADER.to_string(), "true".to_string());
-        hyper_client::get(&provision_url, &headers, None, None, logger::write_warning).await
+        /// Get current GPA service provision status and wait until the GPA service provision finished or timeout
+        /// This function is designed for GPA command line, serves for --status [--wait seconds] option
+        pub async fn get_provision_status_wait(&self) -> ProvisionState {
+            let mut first_loop = true;
+            loop {
+                // query the current provision state from GPA service via http request
+                // ask GPA service listener to notify the its key_keeper in the first loop only
+                let state = match self.get_current_provision_status(first_loop).await {
+                    Ok(state) => state,
+                    Err(e) => {
+                        println!(
+                            "Failed to query the current provision state with error: {}.",
+                            e
+                        );
+                        ProvisionState::new(false, String::new())
+                    }
+                };
+                first_loop = false;
+                if state.finished {
+                    return state;
+                }
+
+                if let Some(d) = self.wait_duration {
+                    if d.as_millis() >= helpers::get_elapsed_time_in_millisec() {
+                        tokio::time::sleep(Duration::from_millis(100)).await;
+                        continue;
+                    }
+                }
+
+                // wait timedout return as 'not finished' with empty message
+                return ProvisionState::new(false, String::new());
+            }
+        }
+
+        // Get current provision status from GPA service via http request
+        // return value
+        //  bool - true provision finished; false provision not finished
+        //  String - provision error message, empty means provision success or provision failed.
+        async fn get_current_provision_status(&self, notify: bool) -> Result<ProvisionState> {
+            let provision_url: String = format!(
+                "http://{}:{}{}",
+                Ipv4Addr::LOCALHOST,
+                self.port,
+                PROVISION_URL_PATH
+            );
+
+            let provision_url: hyper::Uri = provision_url
+                .parse::<hyper::Uri>()
+                .map_err(|e| Error::ParseUrl(provision_url, e.to_string()))?;
+
+            let mut headers = HashMap::new();
+            headers.insert(constants::METADATA_HEADER.to_string(), "true".to_string());
+            headers.insert(
+                constants::TIME_TICK_HEADER.to_string(),
+                self.query_time_tick.to_string(),
+            );
+            if notify {
+                headers.insert(constants::NOTIFY_HEADER.to_string(), "true".to_string());
+            }
+            hyper_client::get(&provision_url, &headers, None, None, logger::write_warning).await
+        }
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::common::logger;
+    use crate::provision::provision_query::ProvisionQuery;
     use crate::provision::ProvisionFlags;
-    use crate::proxy::proxy_connection::ConnectionLogger;
     use crate::proxy::proxy_server;
     use crate::shared_state::SharedState;
-    use proxy_agent_shared::logger_manager;
     use std::env;
     use std::fs;
     use std::time::Duration;
@@ -609,16 +669,6 @@ mod tests {
         // clean up and ignore the clean up errors
         _ = fs::remove_dir_all(&temp_test_path);
 
-        logger_manager::init_logger(
-            logger::AGENT_LOGGER_KEY.to_string(), // production code uses 'Agent_Log' to write.
-            temp_test_path.clone(),
-            logger_key.to_string(),
-            10 * 1024 * 1024,
-            20,
-        )
-        .await;
-        ConnectionLogger::init_logger(temp_test_path.to_path_buf()).await;
-
         // start listener, the port must different from the one used in production code
         let shared_state = SharedState::start_all();
         let cancellation_token = shared_state.get_cancellation_token();
@@ -640,7 +690,7 @@ mod tests {
         let sleep_duration = Duration::from_millis(100);
         tokio::time::sleep(sleep_duration).await;
 
-        let provision_query = super::ProvisionQuery::new(port, None);
+        let provision_query = ProvisionQuery::new(port, None);
         let provision_status = provision_query.get_provision_status_wait().await;
         assert!(
             !provision_status.finished,
@@ -688,8 +738,11 @@ mod tests {
         for handle in handles {
             handle.await;
         }
+        _ = key_keeper_shared_state
+            .update_current_secure_channel_state(super::DISABLE_STATE.to_string())
+            .await;
 
-        let provisioned_file = temp_test_path.join("provisioned.tag");
+        let provisioned_file = temp_test_path.join(super::PROVISION_TAG_FILE_NAME);
         assert!(provisioned_file.exists());
 
         let status_file = temp_test_path.join(super::STATUS_TAG_FILE_NAME);
@@ -700,9 +753,24 @@ mod tests {
             "success status.tag file must be empty"
         );
 
-        let provision_query = super::ProvisionQuery::new(port, Some(Duration::from_millis(5)));
+        let provision_query = ProvisionQuery::new(port, Some(Duration::from_millis(5)));
+        let provision_state_internal = super::get_provision_state_internal(
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+            key_keeper_shared_state.clone(),
+        )
+        .await;
+        assert!(
+            provision_state_internal.finished_time_tick > 0,
+            "finished_time_tick must great than 0"
+        );
+        assert!(!provision_state_internal.is_secure_channel_latched());
+        assert!(
+            provision_state_internal.finished_time_tick < provision_query.get_query_time_tick(),
+            "finished_time_tick must older than the query time_tick"
+        );
         let provision_status = provision_query.get_provision_status_wait().await;
-        assert!(provision_status.finished, "provision_status.0 must be true");
+        assert!(!provision_status.finished, "provision_status.0 must be false as secured channel is disabled and provision finished ");
         assert_eq!(
             0,
             provision_status.errorMessage.len(),
@@ -715,10 +783,55 @@ mod tests {
             .unwrap();
         assert!(event_threads_initialized);
 
+        // update provision finish time_tick
+        super::key_latched(
+            cancellation_token.clone(),
+            key_keeper_shared_state.clone(),
+            telemetry_shared_state.clone(),
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+        )
+        .await;
+        let provision_state_internal = super::get_provision_state_internal(
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+            key_keeper_shared_state.clone(),
+        )
+        .await;
+        assert!(
+            provision_state_internal.finished_time_tick > 0,
+            "finished_time_tick must great than 0"
+        );
+        assert!(!provision_state_internal.is_secure_channel_latched());
+        assert!(
+            provision_state_internal.finished_time_tick > provision_query.get_query_time_tick(),
+            "finished_time_tick must later than the query time_tick"
+        );
+        let provision_status = provision_query.get_provision_status_wait().await;
+        assert!(
+            provision_status.finished,
+            "provision_status.finished must be true as provision finished_time_tick refreshed"
+        );
+        assert_eq!(
+            0,
+            provision_status.errorMessage.len(),
+            "provision_status.1 must be empty"
+        );
+
         // test reset key latch provision state
         super::key_latch_ready_state_reset(provision_shared_state.clone()).await;
         let provision_state = provision_shared_state.get_state().await.unwrap();
         assert!(!provision_state.contains(ProvisionFlags::KEY_LATCH_READY));
+        let provision_state_internal = super::get_provision_state_internal(
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+            key_keeper_shared_state.clone(),
+        )
+        .await;
+        assert!(
+            provision_state_internal.finished_time_tick == 0,
+            "finished_time_tick must be 0 as key latch provision state reset"
+        );
         let provision_status = provision_query.get_provision_status_wait().await;
         assert!(
             !provision_status.finished,
@@ -757,4 +870,88 @@ mod tests {
         // clean up and ignore the clean up errors
         _ = fs::remove_dir_all(&temp_test_path);
     }
+
+    #[tokio::test]
+    async fn provision_status_tag_file_test() {
+        let mut temp_test_path = env::temp_dir();
+        let logger_key = "provision_status_tag_file_test";
+        temp_test_path.push(logger_key);
+        // clean up and ignore the clean up errors
+        _ = fs::remove_dir_all(&temp_test_path);
+
+        let shared_state = SharedState::start_all();
+        let cancellation_token = shared_state.get_cancellation_token();
+        let provision_shared_state = shared_state.get_provision_shared_state();
+        let key_keeper_shared_state = shared_state.get_key_keeper_shared_state();
+        let telemetry_shared_state = shared_state.get_telemetry_shared_state();
+        let agent_status_shared_state = shared_state.get_agent_status_shared_state();
+
+        // test all 3 provision states as ready
+        super::update_provision_state(
+            ProvisionFlags::LISTENER_READY,
+            Some(temp_test_path.clone()),
+            cancellation_token.clone(),
+            key_keeper_shared_state.clone(),
+            telemetry_shared_state.clone(),
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+        )
+        .await;
+        super::update_provision_state(
+            ProvisionFlags::KEY_LATCH_READY,
+            Some(temp_test_path.clone()),
+            cancellation_token.clone(),
+            key_keeper_shared_state.clone(),
+            telemetry_shared_state.clone(),
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+        )
+        .await;
+        super::update_provision_state(
+            ProvisionFlags::REDIRECTOR_READY,
+            Some(temp_test_path.clone()),
+            cancellation_token.clone(),
+            key_keeper_shared_state.clone(),
+            telemetry_shared_state.clone(),
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+        )
+        .await;
+
+        let provisioned_file = temp_test_path.join(super::PROVISION_TAG_FILE_NAME);
+        assert!(provisioned_file.exists());
+        let status_file = temp_test_path.join(super::STATUS_TAG_FILE_NAME);
+        assert!(status_file.exists());
+        assert_eq!(
+            0,
+            status_file.metadata().unwrap().len(),
+            "success status.tag file must be empty"
+        );
+
+        // test key_latch not ready, and error status message contains xml escape characters
+        super::key_latch_ready_state_reset(provision_shared_state.clone()).await;
+        _ = agent_status_shared_state
+            .set_module_status_message(
+                "keyLatchStatus - Failed to acquire key details: Key(KeyResponse(\"acquire\", 403))".to_string(),
+                super::AgentStatusModule::KeyKeeper,
+            )
+            .await;
+        super::provision_timeup(
+            Some(temp_test_path.clone()),
+            provision_shared_state.clone(),
+            agent_status_shared_state.clone(),
+        )
+        .await;
+        //let status_file = temp_test_path.join(super::STATUS_TAG_FILE_NAME);
+        assert!(status_file.exists());
+        let status_file_content = fs::read_to_string(&status_file).unwrap();
+        assert_eq!(
+            "keyLatchStatus - keyLatchStatus - Failed to acquire key details: Key(KeyResponse(&quot;acquire&quot;, 403))\r\n",
+            status_file_content
+        );
+
+        cancellation_token.cancel();
+        // clean up and ignore the clean up errors
+        _ = fs::remove_dir_all(&temp_test_path);
+    }
 }
