@@ -20,7 +20,7 @@
 use super::{proxy_connection::ConnectionLogger, Claims};
 use crate::common::logger;
 use crate::key_keeper::key::{AuthorizationItem, AuthorizationRules, Identity, Privilege, Role};
-use proxy_agent_shared::logger_manager::LoggerLevel;
+use proxy_agent_shared::logger::LoggerLevel;
 use proxy_agent_shared::misc_helpers;
 use serde_derive::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
@@ -164,13 +164,13 @@ impl ComputedAuthorizationItem {
 
     pub fn is_allowed(
         &self,
-        logger: ConnectionLogger,
+        logger: &mut ConnectionLogger,
         request_url: hyper::Uri,
         claims: Claims,
     ) -> bool {
         if self.mode == AuthorizationMode::Disabled {
             logger.write(
-                LoggerLevel::Verbose,
+                LoggerLevel::Trace,
                 "Access control is in disabled state, skip....".to_string(),
             );
 
@@ -180,10 +180,10 @@ impl ComputedAuthorizationItem {
         let mut any_privilege_matched = false;
         for privilege in self.privileges.values() {
             let privilege_name = &privilege.name;
-            if privilege.is_match(&logger, &request_url) {
+            if privilege.is_match(logger, &request_url) {
                 any_privilege_matched = true;
                 logger.write(
-                    LoggerLevel::Verbose,
+                    LoggerLevel::Trace,
                     format!("Request matched privilege '{}'.", privilege_name),
                 );
 
@@ -191,9 +191,9 @@ impl ComputedAuthorizationItem {
                     for assignment in assignments {
                         let identity_name = assignment.clone();
                         if let Some(identity) = self.identities.get(&identity_name) {
-                            if identity.is_match(&logger, &claims) {
+                            if identity.is_match(logger, &claims) {
                                 logger.write(
-                                    LoggerLevel::Verbose,
+                                    LoggerLevel::Trace,
                                     format!(
                                         "Request matched privilege '{}' and identity '{}'.",
                                         privilege_name, identity_name
@@ -204,7 +204,7 @@ impl ComputedAuthorizationItem {
                         }
                     }
                     logger.write(
-                        LoggerLevel::Verbose,
+                        LoggerLevel::Trace,
                         format!(
                             "Request matched privilege '{}' but no identity matched.",
                             privilege_name
@@ -212,7 +212,7 @@ impl ComputedAuthorizationItem {
                     );
                 } else {
                     logger.write(
-                        LoggerLevel::Verbose,
+                        LoggerLevel::Trace,
                         format!(
                             "Request matched privilege '{}' but no identity assigned.",
                             privilege_name
@@ -221,7 +221,7 @@ impl ComputedAuthorizationItem {
                 }
             } else {
                 logger.write(
-                    LoggerLevel::Verbose,
+                    LoggerLevel::Trace,
                     format!("Request does not match privilege '{}'.", privilege_name),
                 );
             }
@@ -229,7 +229,7 @@ impl ComputedAuthorizationItem {
 
         if any_privilege_matched {
             logger.write(
-                LoggerLevel::Information,
+                LoggerLevel::Info,
                 "Privilege matched at least once, but no identity matches, deny the access."
                     .to_string(),
             );
@@ -237,7 +237,7 @@ impl ComputedAuthorizationItem {
         }
 
         logger.write(
-            LoggerLevel::Verbose,
+            LoggerLevel::Trace,
             format!(
                 "No privilege matched, fall back to use the default access: {}.",
                 self.defaultAllowed
@@ -254,6 +254,8 @@ pub struct ComputedAuthorizationRules {
     pub imds: Option<ComputedAuthorizationItem>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub wireserver: Option<ComputedAuthorizationItem>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub hostga: Option<ComputedAuthorizationItem>,
 }
 
 #[derive(Serialize, Deserialize, Clone)]
@@ -340,14 +342,13 @@ impl AuthorizationRulesForLogging {
 #[cfg(test)]
 mod tests {
     use super::{AuthorizationRulesForLogging, ComputedAuthorizationRules};
-    use crate::common::logger;
     use crate::key_keeper::key::{
         AccessControlRules, AuthorizationItem, AuthorizationRules, Identity, Privilege, Role,
         RoleAssignment,
     };
     use crate::proxy::authorization_rules::{AuthorizationMode, ComputedAuthorizationItem};
     use crate::proxy::{proxy_connection::ConnectionLogger, Claims};
-    use proxy_agent_shared::{logger_manager, misc_helpers};
+    use proxy_agent_shared::misc_helpers;
     use std::ffi::OsString;
     use std::path::PathBuf;
     use std::str::FromStr;
@@ -357,11 +358,7 @@ mod tests {
         let logger_key = "test_authorization_rules";
         let mut temp_test_path = std::env::temp_dir();
         temp_test_path.push(logger_key);
-        ConnectionLogger::init_logger(temp_test_path.to_path_buf()).await;
-        let test_logger = ConnectionLogger {
-            tcp_connection_id: 0,
-            http_connection_id: 0,
-        };
+        let mut test_logger = ConnectionLogger::new(0, 0);
 
         // Test Enforce Mode
         let access_control_rules = AccessControlRules {
@@ -414,11 +411,11 @@ mod tests {
         };
         // assert the claim is allowed given the rules above
         let url = hyper::Uri::from_str("http://localhost/test/test").unwrap();
-        assert!(rules.is_allowed(test_logger.clone(), url, claims.clone()));
+        assert!(rules.is_allowed(&mut test_logger, url, claims.clone()));
         let relative_url = hyper::Uri::from_str("/test/test").unwrap();
-        assert!(rules.is_allowed(test_logger.clone(), relative_url.clone(), claims.clone()));
+        assert!(rules.is_allowed(&mut test_logger, relative_url.clone(), claims.clone()));
         claims.userName = "test1".to_string();
-        assert!(!rules.is_allowed(test_logger.clone(), relative_url, claims.clone()));
+        assert!(!rules.is_allowed(&mut test_logger, relative_url, claims.clone()));
 
         // Test Audit Mode
         let access_control_rules = AccessControlRules {
@@ -493,9 +490,9 @@ mod tests {
         assert!(!rules.privileges.is_empty());
 
         let url = hyper::Uri::from_str("http://localhost/test/test1").unwrap();
-        assert!(rules.is_allowed(test_logger.clone(), url, claims.clone()));
+        assert!(rules.is_allowed(&mut test_logger, url, claims.clone()));
         let relative_url = hyper::Uri::from_str("/test/test1").unwrap();
-        assert!(rules.is_allowed(test_logger.clone(), relative_url, claims.clone()));
+        assert!(rules.is_allowed(&mut test_logger, relative_url, claims.clone()));
 
         // Test enforce mode, identity not match
         let access_control_rules = AccessControlRules {
@@ -534,9 +531,9 @@ mod tests {
         assert!(!rules.privileges.is_empty());
 
         let url = hyper::Uri::from_str("http://localhost/test?").unwrap();
-        assert!(!rules.is_allowed(test_logger.clone(), url, claims.clone()));
+        assert!(!rules.is_allowed(&mut test_logger, url, claims.clone()));
         let relativeurl = hyper::Uri::from_str("/test?").unwrap();
-        assert!(!rules.is_allowed(test_logger.clone(), relativeurl, claims.clone()));
+        assert!(!rules.is_allowed(&mut test_logger, relativeurl, claims.clone()));
     }
 
     #[tokio::test]
@@ -555,16 +552,6 @@ mod tests {
         }
         misc_helpers::try_create_folder(&temp_test_path).unwrap();
 
-        // init main logger
-        logger_manager::init_logger(
-            logger::AGENT_LOGGER_KEY.to_string(), // production code uses 'Agent_Log' to write.
-            log_dir.clone(),
-            "logger_key".to_string(),
-            10 * 1024 * 1024,
-            20,
-        )
-        .await;
-
         let access_control_rules = AccessControlRules {
             roles: Some(vec![Role {
                 name: "test".to_string(),
@@ -600,10 +587,12 @@ mod tests {
             Some(AuthorizationRules {
                 imds: Some(authorization_item.clone()),
                 wireserver: Some(authorization_item.clone()),
+                hostga: Some(authorization_item.clone()),
             }),
             ComputedAuthorizationRules {
                 imds: Some(computed_authorization_item.clone()),
                 wireserver: Some(computed_authorization_item.clone()),
+                hostga: Some(computed_authorization_item.clone()),
             },
         );
 
