@@ -221,27 +221,20 @@ pub fn report_status_enable_command(
     report_status(status_folder, config_seq_no, &handler_status);
 }
 
-pub async fn start_event_logger(logger_key: &str) {
+pub async fn start_event_logger() {
     logger::write("starting event logger".to_string());
     tokio::spawn({
-        let logger_key = logger_key.to_string();
         async move {
             let interval: std::time::Duration = std::time::Duration::from_secs(60);
             let max_event_file_count: usize = 50;
             let exe_path = misc_helpers::get_current_exe_dir();
             let event_folder =
                 PathBuf::from(get_handler_environment(&exe_path).eventsFolder.to_string());
-            telemetry::event_logger::start(
-                event_folder,
-                interval,
-                max_event_file_count,
-                &logger_key,
-                |_| {
-                    async {
-                        // do nothing
-                    }
-                },
-            )
+            telemetry::event_logger::start(event_folder, interval, max_event_file_count, |_| {
+                async {
+                    // do nothing
+                }
+            })
             .await;
         }
     });
@@ -443,8 +436,6 @@ mod tests {
 
         //Clean up and ignore the clean up errors
         _ = fs::remove_dir_all(&temp_test_path);
-        let log_folder: String = temp_test_path.to_str().unwrap().to_string();
-        super::logger::init_logger(log_folder, "log.txt").await;
         _ = misc_helpers::try_create_folder(&temp_test_path);
 
         // test invalid dir_path
@@ -492,8 +483,8 @@ mod tests {
         //Clean up and ignore the clean up errors
         _ = fs::remove_dir_all(&temp_test_path);
         _ = misc_helpers::try_create_folder(&temp_test_path);
-
         let status_folder: PathBuf = temp_test_path.join("status");
+
         let config_seq_no = "0";
         let expected_status_file: &PathBuf = &temp_test_path.join("status").join("0.status");
 
@@ -514,8 +505,6 @@ mod tests {
 
         //Clean up and ignore the clean up errors
         _ = fs::remove_dir_all(&temp_test_path);
-        let log_folder: String = temp_test_path.to_str().unwrap().to_string();
-        super::logger::init_logger(log_folder, "log.txt").await;
         _ = misc_helpers::try_create_folder(&temp_test_path);
 
         let expected_heartbeat_file: PathBuf = temp_test_path.join("heartbeat.json");
